import os

from celery import Celery
from celery.schedules import crontab
from celery.utils.log import get_task_logger

from .extractors import ExtractorAllURLs 


# Set the default Django settings module for the 'celery' program.
APP_ENV = os.getenv("APP_ENV", "dev")
os.environ.setdefault("DJANGO_SETTINGS_MODULE", f"agenda_culturel.settings.{APP_ENV}")

app = Celery("agenda_culturel")

logger = get_task_logger(__name__)


# Using a string here means the worker doesn't have to serialize
# the configuration object to child processes.
# - namespace='CELERY' means all celery-related configuration keys
#   should have a `CELERY_` prefix.
app.config_from_object("django.conf:settings", namespace="CELERY")

# Load task modules from all registered Django apps.
app.autodiscover_tasks()

def close_import_task(taskid, success, error_message, importer):
    from agenda_culturel.models import BatchImportation

    task = BatchImportation.objects.get(celery_id=taskid)
    task.status = BatchImportation.STATUS.SUCCESS if success else BatchImportation.STATUS.FAILED
    task.nb_initial = importer.get_nb_events()
    task.nb_imported = importer.get_nb_imported_events()
    task.nb_updated = importer.get_nb_updated_events()
    task.nb_removed = importer.get_nb_removed_events()

    fields = ["status", "nb_initial", "nb_updated", "nb_imported", "nb_removed"]
    if not success:
        task.error_message = error_message
        fields.append("error_message")
    task.save(update_fields=fields)


@app.task(bind=True)
def import_events_from_json(self, json):
    from agenda_culturel.models import Event
    from .importation import EventsImporter

    logger.info("Import events from json: {}".format(self.request.id))

    importer = EventsImporter(self.request.id)

    # try:
    success, error_message = importer.import_events(json)

    # finally, close task
    close_import_task(self.request.id, success, error_message, importer)
    """except Exception as e:
        logger.error(e)
        close_import_task(self.request.id, False, e, importer)"""


@app.task(bind=True)
def import_events_from_url(self, source, browsable_url):
    from agenda_culturel.models import Event
    from .importation import EventsImporter

    logger.info("Import events from url: {} {}".format(source, self.request.id))
    
    # first get json
    # TODO

    # then import
    # importer = EventsImporter(self.request.id)
    # success, error_message = importer.import_events(json)


    success = True
    error_message = ""

    # finally, close task
    close_import_task(self.request.id, success, error_message)



app.conf.timezone = "Europe/Paris"
